uses Dialogs, SysUtils, Classes, Variants, XMLChecker;

  //       
  //      ,  AddErrorToList = False
  procedure OnXMLError(XMLError: IchkXMLError; var AddErrorToList: Boolean);
  begin
    //         
    //    
    if (XMLError.Code = 101) and (XMLError.XMLObject <> nil) and 
      (XMLError.XMLObject.ObjectType = chkxmlNeighbor) then
      XMLError.Kind := chkekWarning;
  end;

  //     XML 
  procedure CheckActiveXML(App: IchkApplication);
  const
    //   ,      Messages.lng
    sErrorNoValueInList = ' "%s"  "%s"       "%s"';
    sErrorWrongParcelParts = '     100%%.   %s%%,   %s';
    sErrorNoParcel = 'XML    ';
    sErrorNoOpenXML = '    XML   ';
    sCaptionError = '';
  
  var
    I: Integer;
    ExchangeFile: IchkUkrainianCadastralExchangeFile;
    ActiveTab: IchkXMLTab;
    
    //      
    function GetParcelPartPercent(Proprietor: IchkXMLCompanyOrPerson): Double;
    var
      Percent, Numerator, 
      Denominator: WideString;
    begin
      Result := 0;
      if not Proprietor.PathExists('ParcelPart') then Exit;
      
      if Proprietor.GetValue('ParcelPart/Percent', Percent) then
        //     
        Result := StrToDoubleSafe(Percent) else
      begin   
        if Proprietor.GetValue('ParcelPart/Part/Numerator', Numerator) and
          Proprietor.GetValue('ParcelPart/Part/Denominator', Denominator) then 
        begin
          //      
          Result := StrToDoubleSafe(Denominator);
          // ,      0
          if Result <> 0 then Result := StrToDoubleSafe(Numerator) / Result * 100;
        end;
      end;
      
    end;
    
    //      ,     100% 
    procedure CheckParcelPart(Parcel: IchkXMLParcel);
    var
      I: Integer;
      SumPercent: Double;
    begin
    
      if Parcel = nil then Exit;

      //   
      SumPercent := 0;
      for I := 0 to Parcel.ProprietorCount - 1 do
        SumPercent := SumPercent + GetParcelPartPercent(Parcel.Proprietors[I]);
      
      if (SumPercent <> 0) and (Round(SumPercent) <> 100) then
      
        //       100%
        ExchangeFile.GenerateError(
          chkegLogicError, //   
          chkekError,           //   
          1,                    //  
          Parcel.XPath, //    XML  
          SafeFormat(App.Translate(sErrorWrongParcelParts), //   
            [IntToStr(Round(SumPercent)), IntToStr(Parcel.ProprietorCount)]));
      
    end;

    //   Value  XMLObject
    //       ListFileName
    // Path -       Value
    // ColumnIndex -      ListFileName
    procedure CheckValueInList(XMLObject: IchkXMLObject; Path, Value, ListFileName: string; 
      ColumnIndex: Integer = 0);
    begin
      if not App.IsValueInList(Value,
        App.GetListFolderPath + '\' + ListFileName, ColumnIndex) then
            
          //      ,  
          ExchangeFile.GenerateError(
            chkegAdditionalError, //   
            chkekError,           //   
            1,                    //  
            XMLObject.XPath + '/' + Path, //    XML  
            SafeFormat(App.Translate(sErrorNoValueInList), //   
              [Value, Path, ListFileName]));
    end;          
  
    //    TagPath  XMLObject 
    //       ListFileName
    // ColumnIndex -      ListFileName
    procedure CheckTagValue(XMLObject: IchkXMLObject; TagPath, ListFileName: string; 
      ColumnIndex: Integer = 0);
    var
      I: Integer;
      TagValue: WideString;
    begin
      
      //      TagValue
      if (XMLObject = nil) or not XMLObject.GetValue(TagPath, TagValue) then Exit;
    
      //       -
      CheckValueInList(XMLObject, TagPath, TagValue, 
        ListFileName, ColumnIndex);
             
    end;  
  
  var
    LandParcel: IchkXMLLandParcel;
    Restriction: IchkXMLRestriction;
    Proprietor: IchkXMLCompanyOrPerson;
    TagValues: IchkStringList;
    FileTab: IchkFileTab;
    Path: string;
  begin

    if App = nil then Exit;
    
    //    XML 
    FileTab := App.MainWindow.FileTabs.ActiveTab;
    if (FileTab = nil) or (FileTab.TabKind <> chktabXML) or
      (FileTab.AsXMLTab.XMLDocument = nil) then
    begin
      // XML  ,  
      App.DisplayHint(chkikError, App.Translate(sCaptionError), App.Translate(sErrorNoOpenXML));
      //   
      Abort;     
    end;
   
    //     
    ActiveTab := FileTab.AsXMLTab;

    // -   
    ExchangeFile := App.CreateUkrainianCadastralExchangeFile;
    //  XML     
    ExchangeFile.LoadFromXMLDocument(ActiveTab.XMLDocument);
    
    //  ,    XML 
    ExchangeFile.OnXMLError := OnXMLError;
  
    //  XML  
    ExchangeFile.ValidateByXSDSchema(App.GetDefaultXSDSchemaFilePath);
    //    ( =     ..)
    ExchangeFile.CheckAreaBalance;
    //   
    ExchangeFile.CheckGeometry;
  
    // XML        
    if ExchangeFile.Parcel <> nil then
    begin
    
      //     
      // Region
      CheckTagValue(ExchangeFile.Parcel, 'ParcelLocationInfo/Region', 'Regions.txt', 1);
      // District
      CheckTagValue(ExchangeFile.Parcel, 'ParcelLocationInfo/District', 'Districts.txt', 1);
      // StreetType
      CheckTagValue(ExchangeFile.Parcel, 'ParcelLocationInfo/ParcelAddress/StreetType', 'StreetTypes.txt');
      // DocumentationType
      CheckTagValue(ExchangeFile.Parcel, 'TechnicalDocumentationInfo/DocumentationType', 'DocumentationType.txt');
      // ExpertOpinion
      CheckTagValue(ExchangeFile.Parcel, 'TechnicalDocumentationInfo/Expertise/ExpertOpinion', 'ExpertOpinion.txt');
      // Category
      CheckTagValue(ExchangeFile.Parcel, 'CategoryPurposeInfo/Category', 'Category.txt');
      // Purpose
      CheckTagValue(ExchangeFile.Parcel, 'CategoryPurposeInfo/Purpose', 'Purpose.txt');
      // LegalModeType
      CheckTagValue(ExchangeFile.Parcel, 'LegalModeInfo/LegalModeType', 'LegalModeType.txt');
      // Code
      CheckTagValue(ExchangeFile.Parcel, 'OwnershipInfo/Code', 'OwnershipCode.txt');
      // StateActType
      CheckTagValue(ExchangeFile.Parcel, 'StateActInfo/StateActType', 'StateActType.txt');

      // DocumentList
      if ExchangeFile.Parcel.GetValues('TechnicalDocumentationInfo/DocumentList', TagValues) then
        for I := 0 to TagValues.Count - 1 do
          //     
          //         
          //  
          if Length(TagValues[I]) = 3 then
          begin
            Path := AddTagIndex('TechnicalDocumentationInfo/DocumentList', I + 1);
            CheckValueInList(ExchangeFile.Parcel, Path, TagValues[I], 'DocumentList.txt');
          end;  
    
      //   
      for I := 0 to ExchangeFile.Parcel.LandParcelCount - 1 do
      begin
        LandParcel := ExchangeFile.Parcel.LandParcels[I];
        // LandCode
        CheckTagValue(LandParcel, 'LandCode', 'LandCode.txt');
      end;

      //   
      for I := 0 to ExchangeFile.Parcel.RestrictionCount - 1 do
      begin
        Restriction := ExchangeFile.Parcel.Restrictions[I];
        // RestrictionCode
        CheckTagValue(Restriction, 'RestrictionCode', 'RestrictionCode.txt');
        // DocumentType
        CheckTagValue(Restriction, 'RestrictionEntitlement/DocumentType', 'RestrictionEntitlementDocumentType.txt');
      end;

      //   
      for I := 0 to ExchangeFile.Parcel.ProprietorCount - 1 do
      begin
        Proprietor := ExchangeFile.Parcel.Proprietors[I];
        // Document
        CheckTagValue(Proprietor, 'PropertyAcquisitionJustification/Document', 'PropertyAcquisitionJustificationDocument.txt');
        // Country (NaturalPerson)
        CheckTagValue(Proprietor, 'Authentication/NaturalPerson/Address/Country', 'Countries.txt');
        // Region (NaturalPerson)
        CheckTagValue(Proprietor, 'Authentication/NaturalPerson/Address/Region', 'Regions.txt', 1);
        // District (NaturalPerson)
        CheckTagValue(Proprietor, 'Authentication/NaturalPerson/Address/District', 'Districts.txt', 1);
        // Country (LegalEntity)
        CheckTagValue(Proprietor, 'Authentication/LegalEntity/Address/Country', 'Countries.txt');
        // Region (LegalEntity)
        CheckTagValue(Proprietor, 'Authentication/LegalEntity/Address/Region', 'Regions.txt', 1);
        // District (LegalEntity)
        CheckTagValue(Proprietor, 'Authentication/LegalEntity/Address/District', 'Districts.txt', 1);
      end;
    
      //     
      CheckParcelPart(ExchangeFile.Parcel);
      
    end else
   
      // XML    
      ExchangeFile.GenerateError(
        chkegLogicError, //   
        chkekError,      //   
        2,               //  
        '',              //    XML  
        App.Translate(sErrorNoParcel));
             
    
    //      
    ActiveTab.Errors := ExchangeFile.Errors;
    //   
    App.MainWindow.Update;
  
  end;
  
var
  App: IchkApplication;
begin

  //      
  //       1.1
  CheckScriptVersion(1.1);
  
  //  -
  App := CreateApplication;
  //   
  App.InitLocalizer(App.GetThisScriptFolder + '\' + 'Messages.lng', 'ru');

  //     XML
  CheckActiveXML(App);
  
end.  