uses Dialogs, SysUtils, Classes, Variants, PaxCompilerFramework, XMLChecker;
 
  //     In4   XML
  procedure ConvertActiveIn4ToXML(App: IchkApplication);
  const
    //   ,      Messages.lng
    sErrorNoOpenIn4 = '    In4    ';
    sErrorIn4ManyParcels = 'In4   ,     In4';
    sCaptionError = '';

    //  In4   dd.mm.yyyy  XML  yyyy-mm-dd
    function In4ToXMLDate(In4Date: string): string;
    var
      Parts: TStringDynArray;
    begin
      Parts := SeparateString(In4Date, '.');
      if Length(Parts) >= 3 then
        Result := Parts[2] + '-' + Parts[1] + '-' + Parts[0] else
        Result := '';
    end;

    //    DescrName  In4Object
    // DescrPart -   
    // ProprietorNumber -   (   |)
    function GetIn4DescriptorValue(In4Object: IchkIn4Object; DescrName: string;
      DescrPart: Integer = 0; ProprietorNumber: Integer = 1): string;
    var
      S: string;
      Descr: IchkIn4Descriptor;
      Props: TStringDynArray;
    begin
      //   ,    
      Result := '';
      if In4Object = nil then Exit;
      //   - In4    
      Descr := In4Object.DescriptorByName(DescrName);
      //      ,  nil
      //        
      //  , NN="1,2,3"
      if (Descr <> nil) and (DescrPart <= Descr.PartCount) then
      begin
        if DescrPart = 0 then
          S := Descr.Value else
          S := Descr.Parts[DescrPart - 1];
        //    ,      |
        // , NN="1-1|1-2,2-1|2-2"
        Props := SeparateString(S, '|');
        if ProprietorNumber > Length(Props) then
          ProprietorNumber := 1;
        if Length(Props) > 0 then
        begin
          Result := Props[ProprietorNumber - 1];
          if Result = '-' then Result := '';
        end;
      end;
    end;

    //  -   DescrName  In4Object
    function GetIn4DescriptorPartCount(In4Object: IchkIn4Object; DescrName: string): Integer;
    var
      Descr: IchkIn4Descriptor;
    begin
      Result := 0;
      if In4Object = nil then Exit;
      Descr := In4Object.DescriptorByName(DescrName);
      if Descr <> nil then Result := Descr.PartCount;
    end;

    //    (- ,  |)
    function GetIn4DescriptorProprietorCount(In4Object: IchkIn4Object;
      DescrName: string; DescrPart: Integer = 1): Integer;
    var
      Descr: IchkIn4Descriptor;
      Props: TStringDynArray;
    begin
      Result := 0;
      if In4Object = nil then Exit;
      Descr := In4Object.DescriptorByName(DescrName);
      if (Descr <> nil) and (DescrPart <= Descr.PartCount) then
      begin
        Props := SeparateString(Descr.Parts[DescrPart - 1], '|');
        Result := Length(Props);
      end;
    end;

    //  True   Value    ColumnIndex   ListFile
    function CheckValueInList(Value, ListFile: string; ColumnIndex: Integer = 0): Boolean;
    begin
      Result := App.IsValueInList(Value, App.GetListFolderPath + '\' + ListFile, ColumnIndex);
    end;

    // True       Phrase     ListFile
    function IsAnyWordInList(Phrase, ListFile: string; ColumnIndex: Integer = 0): Boolean;
    var
      I: Integer;
      S: string;
      Words: TStringDynArray;
    begin
      Result := False;
      //      
      Words := SeparateString(Phrase, ' ');
      for I := 0 to High(Words) do
      begin
        S := Trim(Words[I]);
        if (S <> '') and CheckValueInList(S, ListFile) then
        begin
          Result := True;
          Break;
        end;
      end;
    end;

    //        ListFile    -  Key
    function GetValueByKey(Key, ListFile: string): string;
    begin
      Result := App.GetListValueByKey(Key, App.GetListFolderPath + '\' + ListFile, '');
    end;

    //         
    //       " i"  ". i"
    // ClearStreetName -      
    function GetStreetTypeAndName(StreetName: string; var ClearStreetName: string): string;
    var
      P: Integer;
      S: string;
    begin
      // TODO       
      Result := '';
      ClearStreetName := StreetName;
      //      ,  ,   
      P := FirstPos([' ', '.'], StreetName);
      if P > 0 then
      begin
        //  
        S := Copy(StreetName, 1, P - 1);
        //       ,  ,
        if CheckValueInList(S, 'StreetTypes.txt') then
          Result := S else
          Result := GetValueByKey(S, 'In4ToXMLStreetTypes.txt');
        //       
        if Result = '' then Result := '';
        //       -  
        Delete(ClearStreetName, 1, P);
        ClearStreetName := Trim(ClearStreetName);
      end;
    end;

    // True      
    //     .     ""
    function GetIsUrban(Settlement: string): Boolean;
    begin
      Result := Pos(' ', AnsiLowerCase(Settlement)) = 0;
    end;

    //     In4 
    // In4Object -  ( In4)     DescrName
    // ProprietorNumber -  ,    |
    procedure ParseAddrDescriptor(In4Object: IchkIn4Object; DescrName: string;
      var CountryCode, ZIP, Region, District, Settlement, StreetType, StreetName,
      Building, Block, BuildingUnit: string; ProprietorNumber: Integer = 1);
    begin
      CountryCode := GetIn4DescriptorValue(In4Object, DescrName, 1, ProprietorNumber);
      ZIP := GetIn4DescriptorValue(In4Object, DescrName, 2, ProprietorNumber);
      Region := GetIn4DescriptorValue(In4Object, DescrName, 3, ProprietorNumber);
      District := GetIn4DescriptorValue(In4Object, DescrName, 4, ProprietorNumber);
      Settlement := GetIn4DescriptorValue(In4Object, DescrName, 5, ProprietorNumber);
      StreetType := GetStreetTypeAndName(GetIn4DescriptorValue(In4Object, DescrName, 6, ProprietorNumber), StreetName);
      Building := GetIn4DescriptorValue(In4Object, DescrName, 7, ProprietorNumber);
      Block := GetIn4DescriptorValue(In4Object, DescrName, 8, ProprietorNumber);
      BuildingUnit := GetIn4DescriptorValue(In4Object, DescrName, 9, ProprietorNumber);
    end;

    // True   S      Parts
    function ContainsStringPart(S: string; Parts: array of string): Boolean;
    var
      I: Integer;
      S2: string;
    begin
      Result := False;
      S2 := AnsiLowerCase(S);
      for I := 0 to High(Parts) do
        if Pos(AnsiLowerCase(Parts[I]), S2) > 0 then
        begin
          Result := True;
          Exit;
        end;
    end;

    //  GetIn4DescriptorValue,    DescrPart = 0,
    //      
    function GetIn4DescriptorValueStrict(In4Object: IchkIn4Object; DescrName: string;
      DescrPart, ProprietorNumber: Integer): string;
    begin
      if DescrPart > 0 then
        Result := GetIn4DescriptorValue(In4Object, DescrName, DescrPart, ProprietorNumber) else
        Result := '';
    end;

    // True  S  
    function IsNumeric(S: string): Boolean;
    var
      I: Integer;
    begin
      Result := False;
      for I := 1 to Length(S) do
        if not (S[I] in ['0'..'9']) then Exit;
      Result := True;
    end;

    //  ,     ,   . 
    procedure ParseAddrDescriptor2(In4Object: IchkIn4Object; DescrName: string;
      var CountryCode, ZIP, Region, District, Settlement, StreetType, StreetName,
      Building, Block, BuildingUnit: string; ProprietorNumber: Integer = 1);
    var
      I: Integer;
      S: string;
    begin

      CountryCode := '';
      ZIP := '';
      Region := '';
      District := '';
      Settlement := '';
      StreetType := '';
      StreetName := '';
      Building := '';
      Block := '';
      BuildingUnit := '';

      for I := 1 to 9 do
      begin

        S := GetIn4DescriptorValue(In4Object, DescrName, I, ProprietorNumber);
        if S = '' then Continue;

        //       . 
        if Settlement <> '' then
        begin
          if StreetName = '' then
            StreetType := GetStreetTypeAndName(S, StreetName) else
          // ,       
          if Building = '' then
            Building := S else
            //    - 
            BuildingUnit := S;
        end;

        if I in [1..5] then
        begin
          if (CountryCode = '') and (I = 1) and IsNumeric(S) and (Length(S) = 3) then
            CountryCode := S else
          if (ZIP = '') and IsNumeric(S) and (Length(S) = 5) then
            ZIP := S else
          if (Region = '') and ContainsStringPart(S, [' ', '', ' ', '.']) then
            Region := S else
          if (District = '') and ContainsStringPart(S, ['-', ' ', ' .']) then
            District := S else
          if (Settlement = '') and ContainsStringPart(S, [' ', ' ', ' ',
            '.', ' ', '.', '.', '-']) then
            Settlement := S;
        end;

      end;

      if CountryCode = '' then  CountryCode := '804';

    end;

    // True     
    function IsPartInFraction(Part: string): Boolean;
    begin
      Result := Pos('/', Part) > 0;
    end;

    //       
    procedure GetPartInFraction(Part: string; var Numerator, Denominator: string);
    var
      Parts: TStringDynArray;
    begin
      Numerator := '';
      Denominator := '';
      Parts := SeparateString(Part, '/');
      if Length(Parts) >= 1 then
        Numerator := Trim(Parts[0]);
      if Length(Parts) >= 2 then
        Denominator := Trim(Parts[1]);
    end;

    //      
    function GetPartInPercent(Part: string): string;
    begin
      Result := TrimString(Part, [' ', '%']);
    end;

    //       
    function ClearNeigbourName(Name: string): string;
    const
      // ,      
      cPrefixes: array[0..2] of string = ('', '', ' ');
    var
      I: Integer;
    begin
      Result := Trim(Name);
      for I := 0 to High(cPrefixes) do
        if Pos(cPrefixes[I], AnsiLowerCase(Result)) = 1 then
        begin
          //   
          Delete(Result, 1, Length(cPrefixes[I]));
          Result := Trim(Result);
          Break;
        end;
    end;

    // True   ,   ,
    //    ".",    
    function IsNeigbourNaturalPerson(Name: string): Boolean;
    var
      Name2: string;
    begin
      Name2 := AnsiLowerCase(ClearNeigbourName(Name));
      Result := (Pos('.', Name2) = 1) or (Pos('"', Name2) = 0);
      if Result and IsAnyWordInList(Name2, 'NotPersonNames.txt') then
        Result := False;
    end;

    //      
    function ClearNaturalPersonName(Name: string): string;
    begin
      Result := ClearNeigbourName(Name);
      if Pos('.', AnsiLowerCase(Result)) = 1 then
      begin
        Delete(Result, 1, 3);
        Result := Trim(Result);
      end;
    end;

    //     ,  
    //       
    function GetNeigbourNames(S: string): TStringDynArray;
    var
      I: Integer;
    begin
      Result := SeparateString(S, ',');
      for I := 0 to High(Result) do
        Result[I] := ClearNeigbourName(Result[I]);
    end;

    //      
    function WriteReceiverInfo(XMLFile: IchkUkrainianCadastralExchangeFile; KOATUU: string): Boolean;
    var
      I: Integer;
      S: string;
      Parts: TStringDynArray;
    begin

      Result := False;
      S := GetValueByKey(Copy(KOATUU, 1, 5), 'In4ToXMLReceiver.txt');
      if S = '' then Exit;
      Parts := SeparateString(S, #9);

      for I := 0 to High(Parts) do
      begin

        S := Trim(Parts[I]);
        case I of

          // 
          0: XMLFile.Root.SetValue('ServiceInfo/ReceiverName', S);

          //  
          1: XMLFile.Root.SetValue('ServiceInfo/ReceiverIdentifier', S);
        end;

        Result := True;

      end;

    end;

    //         
    function WriteExecutorInfo(XMLFile: IchkUkrainianCadastralExchangeFile): Boolean;
    type
      TKeyPathPair = record
        Key, Path: string;
      end;

    const
      cKeyPathPairs: array [0..25] of TKeyPathPair = (
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/CompanyName'),
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/EDRPOU'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/TaxNumber'),
        (Key: ' 糿'; Path: 'AdditionalPart/InfoLandWork/Executor/License/LicenseSeries'),
        (Key: ' 糿'; Path: 'AdditionalPart/InfoLandWork/Executor/License/LicenseNumber'),
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/License/LicenseIssuedDate'),
        (Key: 'ϲ  '; Path: 'AdditionalPart/InfoLandWork/Executor/Chief/ChiefName'),
        (Key: '  '; Path: 'AdditionalPart/InfoLandWork/Executor/Chief/ChiefPosition'),
        (Key: '   '; Path: 'AdditionalPart/InfoLandWork/Executor/Chief/Qualification/QualificationNumber'),
        (Key: '    '; Path: 'AdditionalPart/InfoLandWork/Executor/Chief/Qualification/QualificationDate'),
        (Key: 'ϲ '; Path: 'AdditionalPart/InfoLandWork/Executor/Executor/ExecutorName'),
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/Executor/ExecutorPosition'),
        (Key: '  '; Path: 'AdditionalPart/InfoLandWork/Executor/Executor/Qualification/QualificationNumber'),
        (Key: '   '; Path: 'AdditionalPart/InfoLandWork/Executor/Executor/Qualification/QualificationDate'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/Executor/ContactInfo/Phone'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/Executor/ContactInfo/Fax'),
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/Executor/ContactInfo/Email'),
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/Address/Country'),
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/Address/ZIP'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/Address/Region'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/Address/District'),
        (Key: ' '; Path: 'AdditionalPart/InfoLandWork/Executor/Address/Settlement'),
        (Key: '   '; Path: 'AdditionalPart/InfoLandWork/Executor/Address/Street'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/Address/Building'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/Address/Block'),
        (Key: ''; Path: 'AdditionalPart/InfoLandWork/Executor/Address/BuildingUnit'));

    var
      I: Integer;
      S, Key, Path, FirstName,
      MiddleName, LastName: string;
    begin

      Result := False;
      for I := 0 to High(cKeyPathPairs) do
      begin
        Key := cKeyPathPairs[I].Key;
        Path := cKeyPathPairs[I].Path;

        S := GetValueByKey(Key, 'In4ToXMLExecutor.txt');

        if S <> '' then
        begin
          if (Key = 'ϲ  ') or
             (Key = 'ϲ ') then
          begin
            ParseFullName(S, FirstName, MiddleName, LastName);
            XMLFile.Root.SetValue(Path + '/LastName', LastName);
            XMLFile.Root.SetValue(Path + '/FirstName', FirstName);
            XMLFile.Root.SetValue(Path + '/MiddleName', MiddleName);
          end else
            XMLFile.Root.SetValue(Path, S);

          Result := True;

        end;
      end;
    end;

  type
    //    
    TLegalModeType = (lmProperty, lmLease, lmSubLease, lmConstantUse);

  var
    I, J, K, L: Integer;
    XMLFile: IchkUkrainianCadastralExchangeFile;
    In4File: IchkIn4File;
    In4Object, LegalModeIn4Object: IchkIn4Object;
    In4Parcel: IchkIn4Parcel;
    In4LandParcel: IchkIn4LandParcel;
    XMLLandParcel: IchkXMLLandParcel;
    In4Neighbor: IchkIn4Neighbor;
    XMLNeighbor: IchkXMLNeighbor;
    XMLProprietor, XMLGrantee,
    XMLLeasee, XMLSubLeasee,
    XMLBeneficiary: IchkXMLCompanyOrPerson;
    XMLLease: IchkXMLLease;
    XMLSubLease: IchkXMLSubLease;
    XMLRestriction: IchkXMLRestriction;
    FileTab: IchkFileTab;
    FileName, Path, FirstName, MiddleName, LastName,
    CSIn4Kind, CSXMLName, CSDescr, HSIn4Kind, HSXMLName,
    Region, District,
    Settlement, StreetType,
    StreetName, Building, Block,
    BuildingUnit, KOATUU, OwnershipCode,
    CadastralZoneNumber, CadastralQuarterNumber,
    CadastralParcelNumber, FullCadastralNumber, Use, Category, KF, KU, PV,
    PurposeCode, TempStr, PasspType, PasspNum, PasspSer,
    CountryCode, ZIP,
    DocType, DocDate, DocNumber, Authority,
    StateActType, StateActSer, StateActNum,
    TaxNumber, Valuation, LegalModeCode,
    DescrName, LandCode, ParcelPart,
    Numerator, Denominator, MoneyPayment,
    RestrictionTerm, RestrictionCode, NewRestrictionCode,
    RegistrationDate, RegistrationNumber: string;
    IsUrban, IsNaturalPerson,
    IsValidAddr: Boolean;
    ProprietorCount, RestrictionCount, GranteeCount,
    LeaseeCount, SubLeaseeCount: Integer;
    PasspSerNum, StateActSerNum, NeigbourNames,
    Restrictions, Beneficiaries: TStringDynArray;
    LegalModeType: TLegalModeType;
  begin

    if App = nil then Exit;

    //    In4 
    FileTab := App.MainWindow.FileTabs.ActiveTab;
    if (FileTab = nil) or (FileTab.TabKind <> chktabIn4) or
      (FileTab.AsIn4Tab.In4File = nil) then
    begin
      // In4  ,  
      App.DisplayHint(chkikError, App.Translate(sCaptionError), App.Translate(sErrorNoOpenIn4));
      //   
      Abort;
    end;

    //  In4   
    In4File := FileTab.AsIn4Tab.In4File;

    if In4File.Quarter.ParcelCount <> 1 then
    begin
      // In4        
      App.DisplayHint(chkikError, App.Translate(sCaptionError), App.Translate(sErrorIn4ManyParcels));
      //   
      Abort;
    end;

    // -XML ,     In4
    XMLFile := App.CreateUkrainianCadastralExchangeFile;

    //  ,    In4 
    XMLFile.CadastralZone.Geometry := In4File.Quarter.Geometry;

    // 
    XMLFile.CadastralQuarter.Geometry := In4File.Quarter.Geometry;

    //  In4
    In4Parcel := In4File.Quarter.Parcels[0];
    XMLFile.Parcel.Geometry := In4Parcel.Geometry;

    //   
    if not WriteExecutorInfo(XMLFile) then
    begin
      //   
      XMLFile.Root.SetValue('AdditionalPart/InfoLandWork/Executor/CompanyName',
        GetIn4DescriptorValue(In4Parcel, 'TD', 3));

      //  
      ParseFullName(GetIn4DescriptorValue(In4Parcel, 'TD', 4), FirstName, MiddleName, LastName);
      XMLFile.Root.SetValue('AdditionalPart/InfoLandWork/Executor/Executor/ExecutorName/LastName', LastName);
      XMLFile.Root.SetValue('AdditionalPart/InfoLandWork/Executor/Executor/ExecutorName/FirstName', FirstName);
      XMLFile.Root.SetValue('AdditionalPart/InfoLandWork/Executor/Executor/ExecutorName/MiddleName', MiddleName);
    end;

    //   
    CSIn4Kind := GetIn4DescriptorValue(In4File.Quarter, 'CS', 1);
    CSDescr := GetIn4DescriptorValue(In4File.Quarter, 'CS', 2);

    //  XML     In4 
    if CSIn4Kind = '0' then CSXMLName := 'SC42' else
      if CSIn4Kind = '1' then CSXMLName := 'SC42_3' else
        if CSIn4Kind = '2' then CSXMLName := 'SC63' else
          if CSIn4Kind = '3' then CSXMLName := 'Local' else
            CSXMLName := 'SC63';

    Path := 'InfoPart/MetricInfo/CoordinateSystem/' + CSXMLName;
    if CSXMLName = 'SC63' then
    begin
      //    63,     
      if (CSDescr = '') or not (CSDescr[1] in ['X', 'C', 'P', 'T']) then
        CSDescr := 'X';
      XMLFile.Root.SetValue(Path + '/' + CSDescr[1], '');
    end else
      //     CSDescr       
      XMLFile.Root.SetValue(Path, CSDescr);

    //   
    HSIn4Kind := GetIn4DescriptorValue(In4File.Quarter, 'HS');
    if HSIn4Kind = '0' then HSXMLName := 'Baltic77' else
      if HSIn4Kind = '1' then HSXMLName := 'Baltic' else
        HSXMLName := 'Other';
    XMLFile.Root.SetValue('InfoPart/MetricInfo/HeightSystem/' + HSXMLName, '');

    //   ,  
    XMLFile.Root.SetValue('InfoPart/MetricInfo/MeasurementUnit/M', '');

    // 
    KOATUU := GetIn4DescriptorValue(In4File.Quarter, 'DS');

    //   
    WriteReceiverInfo(XMLFile, KOATUU);

    // 
    //XMLFile.CadastralZone.SetValue('KOATUU', KOATUU);

    //   
    CadastralZoneNumber := GetIn4DescriptorValue(In4File.Quarter, 'SD');
    if (KOATUU<>'') or (CadastralZoneNumber<>'')
      then XMLFile.CadastralZone.SetValue('CadastralZoneNumber', KOATUU+':'+CadastralZoneNumber);

    //   
    CadastralQuarterNumber := GetIn4DescriptorValue(In4File.Quarter, 'BC');
    if CadastralQuarterNumber<>''
      then XMLFile.CadastralQuarter.SetValue('CadastralQuarterNumber', CadastralQuarterNumber);

    //  
    Region := GetValueByKey(Copy(KOATUU, 1, 2), 'Regions.txt');
    District := GetValueByKey(Copy(KOATUU, 1, 5), 'Districts.txt');
    Settlement := GetIn4DescriptorValue(In4Parcel, 'AD', 1);
    StreetType := GetStreetTypeAndName(GetIn4DescriptorValue(In4Parcel, 'AD', 2), StreetName);
    Building := GetIn4DescriptorValue(In4Parcel, 'AD', 3);
    Block := GetIn4DescriptorValue(In4Parcel, 'AD', 4);
    IsUrban := GetIsUrban(Settlement);

    if Region<>''
      then XMLFile.Parcel.SetValue('ParcelLocationInfo/Region', Region);
    if Settlement<>''
      then XMLFile.Parcel.SetValue('ParcelLocationInfo/Settlement', Settlement);
    if District<>''
      then XMLFile.Parcel.SetValue('ParcelLocationInfo/District', District);
    if IsUrban then
      XMLFile.Parcel.SetValue('ParcelLocationInfo/ParcelLocation/Urban', '') else
      XMLFile.Parcel.SetValue('ParcelLocationInfo/ParcelLocation/Rural', '');
    if StreetName <> '' then
    begin
      if StreetType<>''
        then XMLFile.Parcel.SetValue('ParcelLocationInfo/ParcelAddress/StreetType', StreetType);
      if StreetName<>''
        then XMLFile.Parcel.SetValue('ParcelLocationInfo/ParcelAddress/StreetName', StreetName);
      if Building<>''
        then XMLFile.Parcel.SetValue('ParcelLocationInfo/ParcelAddress/Building', Building);
      if Block <> '' then
        XMLFile.Parcel.SetValue('ParcelLocationInfo/ParcelAddress/Block', Block);
    end;

    //  
    Category := GetValueByKey(GetIn4DescriptorValue(In4Parcel, 'CV'), 'In4ToXMLCategories.txt');
    if Category<>''
      then XMLFile.Parcel.SetValue('CategoryPurposeInfo/Category', Category);

    //    
    PurposeCode := GetIn4DescriptorValue(In4Parcel, 'CV');
    if PurposeCode<>''
      then XMLFile.Parcel.SetValue('CategoryPurposeInfo/Purpose', PurposeCode);

    //   
    Use := GetIn4DescriptorValue(In4Parcel, 'TX');
    if Use<>''
      then XMLFile.Parcel.SetValue('CategoryPurposeInfo/Use', Use);

    //   
    OwnershipCode := GetValueByKey(GetIn4DescriptorValue(In4Parcel, 'PF'), 'In4ToXMLOwnershipCode.txt');
    if OwnershipCode<>''
      then XMLFile.Parcel.SetValue('OwnershipInfo/Code', OwnershipCode);

    //   
    CadastralParcelNumber := Copy(GetIn4DescriptorValue(In4Parcel, 'SC'), 4, 4);
    if CadastralParcelNumber<>''
      then XMLFile.Parcel.SetValue('ParcelMetricInfo/ParcelID', CadastralParcelNumber);

    //    
    FullCadastralNumber:=KOATUU + ':' + CadastralZoneNumber + ':' + CadastralQuarterNumber + ':' + CadastralParcelNumber;

    //  
    //XMLFile.Parcel.SetValue('ParcelMetricInfo/Description', Use);

    //  
    KF := GetIn4DescriptorValue(In4Parcel, 'KF');
    KU := GetIn4DescriptorValue(In4Parcel, 'KU');
    //     KF,   KU
    IsNaturalPerson := KF <> '';

    //  
    ProprietorCount := GetIn4DescriptorProprietorCount(In4Parcel, 'NM');

    //    
    PV := AnsiUpperCase(GetIn4DescriptorValue(In4Parcel, 'PV', 1));
    if PV = '' then
      // 
      LegalModeType := lmSubLease else
    if PV = '' then
      // 
      LegalModeType := lmLease else
    if (PV = '') or (PV = '') then
      //  
      LegalModeType := lmConstantUse else
      // 
      LegalModeType := lmProperty;

    //     XML 
    for I := 1 to ProprietorCount do
    begin

      //     XML 
      XMLProprietor := XMLFile.Parcel.AddNewProprietor;

      //  
      ParcelPart := GetIn4DescriptorValue(In4Parcel, 'VP', 1, I);
      if ParcelPart <> '' then
        if IsPartInFraction(ParcelPart) then
        begin
          GetPartInFraction(ParcelPart, Numerator, Denominator);
          XMLProprietor.SetValue('ParcelPart/Part/Numerator', Numerator);
          XMLProprietor.SetValue('ParcelPart/Part/Denominator', Denominator);
        end else
        begin
          Numerator := GetPartInPercent(ParcelPart);
          XMLProprietor.SetValue('ParcelPart/Percent', Numerator);
        end;

      //  
      ParseAddrDescriptor(In4Parcel, 'AU', CountryCode, ZIP, Region, District,
        Settlement, StreetType, StreetName, Building, Block, BuildingUnit, I);

      // 
      if IsNaturalPerson then
      begin

        //  
        ParseFullName(GetIn4DescriptorValue(In4Parcel, 'NM', 0, I), FirstName, MiddleName, LastName);
        XMLProprietor.SetValue('Authentication/NaturalPerson/FullName/LastName', LastName);
        XMLProprietor.SetValue('Authentication/NaturalPerson/FullName/FirstName', FirstName);
        if MiddleName <> '' then
          XMLProprietor.SetValue('Authentication/NaturalPerson/FullName/MiddleName', MiddleName);

        //  
        XMLProprietor.SetValue('Authentication/NaturalPerson/TaxNumber',
          GetIn4DescriptorValue(In4Parcel, 'KF', 0, I));

        //   
        PasspType := GetValueByKey(GetIn4DescriptorValue(In4Parcel, 'PP', 2, I), 'In4ToXMLPassportTypes.txt');
        if PasspType<>''
          then XMLProprietor.SetValue('Authentication/NaturalPerson/Passport/DocumentType', PasspType);

        //    
        PasspSerNum := SeparateString(GetIn4DescriptorValue(In4Parcel, 'PP', 3, I), ' ');
        if Length(PasspSerNum) = 1 then
        begin
          PasspSer := Copy(PasspSerNum[0], 1, 2);
          PasspNum := Copy(PasspSerNum[0], 3, 20);
        end else
        if Length(PasspSerNum) > 1 then
        begin
          PasspSer := PasspSerNum[0];
          PasspNum := PasspSerNum[1];
        end else
        begin
          PasspSer := '';
          PasspNum := '';
        end;
        if PasspNum<>''
          then XMLProprietor.SetValue('Authentication/NaturalPerson/Passport/PassportNumber', PasspNum);
        if PasspSer<>''
          then XMLProprietor.SetValue('Authentication/NaturalPerson/Passport/PassportSeries', PasspSer);

        //   
        CountryCode := GetIn4DescriptorValue(In4Parcel, 'PP', 1, I);
        if CountryCode<>''
          then XMLProprietor.SetValue('Authentication/NaturalPerson/Citizenship', CountryCode);

        Path := 'Authentication/NaturalPerson/Address/';

      end else
      // 
      begin

        //  
        XMLProprietor.SetValue('Authentication/LegalEntity/Name', GetIn4DescriptorValue(In4Parcel, 'NM', 0, I));

        //  
        TempStr := GetIn4DescriptorValue(In4Parcel, 'KU', 0, I);
        if TempStr<>''
          then XMLProprietor.SetValue('Authentication/LegalEntity/EDRPOU', TempStr);

        Path := 'Authentication/LegalEntity/Address/';

      end;

      //  
      if CountryCode<>''
        then XMLProprietor.SetValue(Path + 'Country', CountryCode);
      if ZIP<>''
        then XMLProprietor.SetValue(Path + 'ZIP', ZIP);
      if Region<>''
        then XMLProprietor.SetValue(Path + 'Region', Region);
      if District<>''
        then XMLProprietor.SetValue(Path + 'District', District);
      if Settlement<>''
        then XMLProprietor.SetValue(Path + 'Settlement', Settlement);
      if (StreetType<>'') or (StreetName<>'')
        then XMLProprietor.SetValue(Path + 'Street', Trim(StreetType + ' ' + StreetName));
      if Building<>''
        then XMLProprietor.SetValue(Path + 'Building', Building);
      if Block<>''
        then XMLProprietor.SetValue(Path + 'Block', Block);
      if BuildingUnit<>''
        then XMLProprietor.SetValue(Path + 'BuildingUnit', BuildingUnit);

      //   
      TempStr := GetIn4DescriptorValue(In4Parcel, 'KZ');
      if TempStr<>''
        then XMLProprietor.SetValue('ProprietorCode', TempStr);

      if not (LegalModeType in [lmLease, lmSubLease]) then
      begin

        //    
        DocType := GetValueByKey(GetIn4DescriptorValue(In4Parcel, 'PZ', 1, I), 'In4ToXMLPropertyAcquisitionJustification.txt');
        if DocType<>''
          then XMLProprietor.SetValue('PropertyAcquisitionJustification/Document', DocType);

        //  
        DocDate := In4ToXMLDate(GetIn4DescriptorValue(In4Parcel, 'PZ', 2, I));
        if DocDate<>''
          then XMLProprietor.SetValue('PropertyAcquisitionJustification/DocumentDate', DocDate);

        //  
        DocNumber := GetIn4DescriptorValue(In4Parcel, 'PZ', 3, I);
        if DocNumber<>''
          then XMLProprietor.SetValue('PropertyAcquisitionJustification/DocumentNumber', DocNumber);

        // ,  
        Authority := GetIn4DescriptorValue(In4Parcel, 'PZ', 4, I);
        if Authority<>''
          then XMLProprietor.SetValue('PropertyAcquisitionJustification/ApprovalAuthority', Authority);
      end;

    end;

    //   
    DocType := GetValueByKey(GetIn4DescriptorValue(In4Parcel, 'TD', 1), 'In4ToXMLDocumentationType.txt');
    if DocType<>''
      then XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentationType', DocType);

    //   
    DocDate := In4ToXMLDate(GetIn4DescriptorValue(In4Parcel, 'TD', 2));
    if DocDate<>''
      then XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DraftingDate', DocDate);

    //   ( ,    In4)
    //  
    XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[1]', '001');

    //  
    XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[2]', '002');

    // ﳿ  ()  /       
    XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[3]', '007');

    //    
    XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[4]', '022');

    if IsNaturalPerson then
    begin

      //  ,   
      XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[5]', '041');

      //        
      XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[6]', '042');

    end else
    begin

      //             
      XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[5]', '043');

      //   
      XMLFile.Parcel.SetValue('TechnicalDocumentationInfo/DocumentList[6]', '044');

    end;

    //  
    StateActType := GetValueByKey(GetIn4DescriptorValue(In4Parcel, 'GA', 1), 'In4ToXMLStateActType.txt');
    if StateActType<>''
      then XMLFile.Parcel.SetValue('StateActInfo/StateActType', StateActType);

    //    
    StateActSerNum := SeparateString(GetIn4DescriptorValue(In4Parcel, 'GA', 2), ' ');
    if Length(StateActSerNum) > 1 then
    begin
      StateActSer := StateActSerNum[0];
      StateActNum := StateActSerNum[1];
    end else
    begin
      StateActSer := '';
      if Length(StateActSerNum) > 0 then
        StateActNum := StateActSerNum[0] else
        StateActNum := '';
    end;

    if (StateActType<>'') and (StateActSer<>'')
      then XMLFile.Parcel.SetValue('StateActInfo/StateActForm/Series', StateActSer);
    if (StateActType<>'') and (StateActNum<>'')
      then XMLFile.Parcel.SetValue('StateActInfo/StateActForm/Number', StateActNum);

    //   
    //XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationBookNumber',
    //  GetIn4DescriptorValue(In4Parcel, 'GA', 3));

    //    
    // XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/SectionNumber', '1');

    //  
    TempStr := GetIn4DescriptorValue(In4Parcel, 'GA', 4);
    if (StateActType<>'') and (TempStr<>'')
      then XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationNumber', TempStr);

    //  
    TempStr := GetIn4DescriptorValue(In4Parcel, 'GA', 5);
    if (StateActType<>'') and (TempStr<>'')
      then XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationDate', In4ToXMLDate(TempStr));

    // ,  
    //     . 
    ParseFullName(GetIn4DescriptorValue(In4File.Quarter, 'ZM'), FirstName, MiddleName, LastName);
    XMLFile.CadastralQuarter.SetValue('RegionalContacts/DKZRHead/LastName', LastName);
    XMLFile.CadastralQuarter.SetValue('RegionalContacts/DKZRHead/FirstName', FirstName);
    XMLFile.CadastralQuarter.SetValue('RegionalContacts/DKZRHead/MiddleName', MiddleName);

    if StateActType<>''
      then begin
        XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationPersonInfo/SignedByName[1]/LastName', LastName);
        XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationPersonInfo/SignedByName[1]/FirstName', FirstName);
        XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationPersonInfo/SignedByName[1]/MiddleName', MiddleName);
      end;

    //    /
    ParseFullName(GetIn4DescriptorValue(In4File.Quarter, 'GL'), FirstName, MiddleName, LastName);
    XMLFile.CadastralQuarter.SetValue('RegionalContacts/LocalAuthorityHead/LastName', LastName);
    XMLFile.CadastralQuarter.SetValue('RegionalContacts/LocalAuthorityHead/FirstName', FirstName);
    XMLFile.CadastralQuarter.SetValue('RegionalContacts/LocalAuthorityHead/MiddleName', MiddleName);

    if StateActType<>''
      then begin
        XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationPersonInfo/SignedByName[2]/LastName', LastName);
        XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationPersonInfo/SignedByName[2]/FirstName', FirstName);
        XMLFile.Parcel.SetValue('StateActInfo/StateActRegistrationInfo/RegistrationPersonInfo/SignedByName[2]/MiddleName', MiddleName);
      end;

    //   
    TempStr := GetIn4DescriptorValue(In4Parcel, 'GA', 5);
    if (StateActType<>'') and (TempStr<>'')
      then XMLFile.Parcel.SetValue('StateActInfo/DeliveryDate', In4ToXMLDate(TempStr));

    //   
    Valuation := GetIn4DescriptorValue(In4Parcel, 'TM', 3);
    if (Valuation <> '') and (AnsiUpperCase(GetIn4DescriptorValue(In4Parcel, 'TM', 1)) = '') then
    begin

      //  
      XMLFile.Parcel.SetValue('ValuationInfo/Normative/ValuationDate',
        In4ToXMLDate(GetIn4DescriptorValue(In4Parcel, 'TM', 2)));

      //  
      XMLFile.Parcel.SetValue('ValuationInfo/Normative/Value', Valuation);

    end;

    // 
    if LegalModeType in [lmLease, lmSubLease]  then
    begin

      for I := 0 to In4File.In4ObjectCount - 1 do
      begin
        In4Object := In4File.In4Objects[I];

        //     
        if not (In4Object.ObjectType in [chkin4Parcel, chkin4LandParcel]) or
          (GetIn4DescriptorValue(In4Object, 'LE', 1) = '') then Continue;

        XMLLease := XMLFile.Parcel.AddNewLease;
        XMLLease.Geometry := In4Object.Geometry;

        //    
        DocType := GetValueByKey(GetIn4DescriptorValue(In4Object, 'LE', 3), 'In4ToXMLLeaseApprovalDocument.txt');
        XMLLease.SetValue('ExecutivePowerDecision/ApprovalDocument', DocType);

        //  
        DocDate := In4ToXMLDate(GetIn4DescriptorValue(In4Object, 'LE', 4));
        XMLLease.SetValue('ExecutivePowerDecision/ApprovalDate', DocDate);

        //  
        XMLLease.SetValue('ExecutivePowerDecision/ApprovalDocumentNumber',
          GetIn4DescriptorValue(In4Object, 'LE', 5));

        // - 
        LeaseeCount := GetIn4DescriptorProprietorCount(In4Object,'LE',1);
        for J := 1 to LeaseeCount do
        begin
          XMLLeasee := XMLLease.AddNewLeasee;

          IsValidAddr := GetIn4DescriptorPartCount(In4Parcel, 'CM') = 9;
          if IsValidAddr then
          begin
            //      In4 
            ParseAddrDescriptor(In4Parcel, 'CM', CountryCode, ZIP, Region, District,
              Settlement, StreetType, StreetName, Building, Block, BuildingUnit, J);
          end else
          begin
            //  - 
            ParseAddrDescriptor2(In4Parcel, 'CM', CountryCode, ZIP, Region, District,
            Settlement, StreetType, StreetName, Building, Block, BuildingUnit, J);
          end;

          //  ,   
          TaxNumber := GetIn4DescriptorValue(In4Object, 'LE', 2, J);
          IsNaturalPerson := Length(TaxNumber) = 10;

          // 
          if IsNaturalPerson then
          begin

            Path := 'NaturalPerson/';

            //  
            ParseFullName(GetIn4DescriptorValue(In4Object, 'LE', 1, J),
              FirstName, MiddleName, LastName);
            XMLLeasee.SetValue(Path + 'FullName/LastName', LastName);
            XMLLeasee.SetValue(Path + 'FullName/FirstName', FirstName);
            if MiddleName<>''
              then XMLLeasee.SetValue(Path + 'FullName/MiddleName', MiddleName);

            //  
            if TaxNumber<>''
              then XMLLeasee.SetValue(Path + 'TaxNumber', TaxNumber);

          end else
          begin

            Path := 'LegalEntity/';

            //  -
            XMLLeasee.SetValue(Path + 'Name', GetIn4DescriptorValue(LegalModeIn4Object, 'LE', 1, I));

            //  
            if TaxNumber<>''
              then XMLLeasee.SetValue(Path + 'EDRPOU', TaxNumber);

          end;

          //   ,    
          if CountryCode<>''
            then XMLLeasee.SetValue(Path + 'Address/Country', CountryCode);
          if ZIP<>''
            then XMLLeasee.SetValue(Path + 'Address/ZIP', ZIP);
          if Region<>''
            then XMLLeasee.SetValue(Path + 'Address/Region', Region);
          if District<>''
            then XMLLeasee.SetValue(Path + 'Address/District', District);
          if Settlement<>''
            then XMLLeasee.SetValue(Path + 'Address/Settlement', Settlement);
          if (StreetType<>'') or (StreetName<>'')
            then XMLLeasee.SetValue(Path + 'Address/Street', Trim(StreetType + ' ' + StreetName));
          if Building<>''
            then XMLLeasee.SetValue(Path + 'Address/Building', Building);
          if Block<>''
            then XMLLeasee.SetValue(Path + 'Address/Block', Block);
          if BuildingUnit<>''
            then XMLLeasee.SetValue(Path + 'Address/BuildingUnit', BuildingUnit);

          //   ,      . 
          if Settlement = '' then
            XMLLeasee.SetValue(Path + 'AdditionalInfoBlock/AdditionalInfo',
              GetIn4DescriptorValue(In4Parcel, 'CM', 0, I));

        end;

        //   
        TempStr := GetIn4DescriptorValue(In4Object, 'LE', 9);
        if TempStr<>''
          then XMLLease.SetValue('LeaseAgreement/Area', TempStr);

        //    
        TempStr := GetIn4DescriptorValue(In4Object, 'PV', 2);
        if TempStr<>''
          then XMLLease.SetValue('LeaseAgreement/LeaseTerm/LeaseDuration', TempStr);

        //     
        TempStr := GetIn4DescriptorValue(In4Object, 'LE', 10);
        if TempStr<>''
          then XMLLease.SetValue('LeaseAgreement/Rent/MoneyRent', TempStr);

        //     
        TempStr := GetIn4DescriptorValue(In4Object, 'LE', 13);
        if TempStr<>''
          then XMLLease.SetValue('LeaseAgreement/Rent/OtherRent', TempStr);

        //    
        TempStr := GetIn4DescriptorValue(In4Object, 'LE', 7);
        if TempStr<>''
          then XMLLease.SetValue('LeaseAgreement/RegistrationNumber', TempStr);

        //    
        TempStr := GetIn4DescriptorValue(In4Object, 'LE', 6);
        if TempStr<>''
          then XMLLease.SetValue('LeaseAgreement/RegistrationDate', In4ToXMLDate(TempStr));

      end;
    end;

    // 
    if LegalModeType = lmSubLease  then
    begin
      for I := 0 to In4File.In4ObjectCount - 1 do
      begin
        In4Object := In4File.In4Objects[I];

        //     
        if not (In4Object.ObjectType in [chkin4Parcel, chkin4LandParcel]) or
          (GetIn4DescriptorValue(In4Object, 'SL', 1) = '') then Continue;

        XMLSubLease := XMLFile.Parcel.AddNewSubLease;
        XMLSubLease.Geometry := In4Object.Geometry;

        // - 
        SubLeaseeCount := GetIn4DescriptorProprietorCount(In4Object,'SL',1);
        for J := 1 to SubLeaseeCount do
        begin
          XMLSubLeasee := XMLSubLease.AddNewSubLeasee;

          //  ,   
          TaxNumber := GetIn4DescriptorValue(In4Object, 'SL', 2, J);
          IsNaturalPerson := Length(TaxNumber) = 10;

          // 
          if IsNaturalPerson then
          begin

            Path := 'NaturalPerson/';

            //  
            ParseFullName(GetIn4DescriptorValue(In4Object, 'SL', 1, J),
              FirstName, MiddleName, LastName);
            XMLSubLeasee.SetValue(Path + 'FullName/LastName', LastName);
            XMLSubLeasee.SetValue(Path + 'FullName/FirstName', FirstName);
            if MiddleName<>''
              then XMLSubLeasee.SetValue(Path + 'FullName/MiddleName', MiddleName);

            //  
            if TaxNumber<>''
              then XMLSubLeasee.SetValue(Path + 'TaxNumber', TaxNumber);

          end else
          begin

            Path := 'LegalEntity/';

            //  -
            XMLSubLeasee.SetValue(Path + 'Name', GetIn4DescriptorValue(In4Object, 'SL', 1, J));

            //  
            if TaxNumber<>''
              then XMLSubLeasee.SetValue(Path + 'EDRPOU', TaxNumber);

          end;
        end;

        //   
        TempStr := GetIn4DescriptorValue(In4Object, 'SL', 6);
        if TempStr<>''
          then XMLSubLease.SetValue('Area', TempStr);

        //    
        TempStr := GetIn4DescriptorValue(In4Object, 'SL', 3);
        if TempStr<>''
          then XMLSubLease.SetValue('RegistrationDate', In4ToXMLDate(TempStr));

        //    
        TempStr := GetIn4DescriptorValue(In4Object, 'SL', 4);
        if TempStr<>''
          then XMLSubLease.SetValue('RegistrationNumber', TempStr);

        //     
        TempStr := GetIn4DescriptorValue(In4Object, 'SL', 7);
        if TempStr<>''
          then XMLSubLease.SetValue('Subrent/MoneySubrent', TempStr);

        //     
        TempStr := GetIn4DescriptorValue(In4Object, 'SL', 10);
        if TempStr<>''
          then XMLSubLease.SetValue('Subrent/OtherSubrent', TempStr);

        //    
        XMLSubLease.SetValue('SubleaseTerm/SubleaseDuration',
          GetIn4DescriptorValue(In4Parcel, 'PV', 2));

      end;
    end;

    //   -  
    if LegalModeType = lmConstantUse then
    begin
      //  ,    
      XMLFile.Parcel.SetValue('LegalModeInfo/LegalModeType', '1');

      //   
      for I := 0 to ProprietorCount - 1 do
      begin
        //  -  
        XMLGrantee := XMLFile.Parcel.AddNewGrantee;

        //        
        XMLFile.CopyValues(XMLFile.Parcel.Proprietors[I], 'Authentication',
          XMLGrantee, '');
      end;

      //      ,   
      XMLFile.Parcel.SetValue('LegalModeInfo/Grantor/LegalEntity/Name',
        GetValueByKey(Copy(KOATUU, 1, 8), 'Councils.txt'));
    end;

    //   - 
    if LegalModeType = lmLease then
    begin
      //  ,   
      XMLFile.Parcel.SetValue('LegalModeInfo/LegalModeType', '2');

      //   
      if XMLFile.Parcel.LeaseCount>0 then
        for I := 0 to XMLFile.Parcel.Leases[0].LeaseeCount - 1 do
        begin
          //  -  
          XMLGrantee := XMLFile.Parcel.AddNewGrantee;

          //        
          XMLFile.CopyValues(XMLFile.Parcel.Leases[0].Leasees[I], '', XMLGrantee, '');
        end;

      //     1-    ,   
      //if XMLFile.Parcel.ProprietorCount>0 then
      //  XMLFile.CopyValues(XMLFile.Parcel.Proprietors[0], 'Authentication',
        //   XMLFile.Parcel, 'LegalModeInfo/Grantor');
    end;

    //   - 
    if LegalModeType = lmSublease then
    begin
      //  ,   
      XMLFile.Parcel.SetValue('LegalModeInfo/LegalModeType', '3');

      //   
      if XMLFile.Parcel.SubLeaseCount>0 then
      for I := 0 to XMLFile.Parcel.SubLeases[0].SubLeaseeCount - 1 do
      begin
        //  -  
        //XMLGrantee := XMLFile.Parcel.AddNewGrantee;

        //        
        //XMLFile.CopyValues(XMLFile.Parcel.SubLeases[0].SubLeasees[I], '', XMLGrantee, '');
      end;

      //     1-    ,   
      //if (XMLFile.Parcel.LeaseCount>0) and (XMLFile.Parcel.Leases[0].LeaseeCount>0) then
        //XMLFile.CopyValues(XMLFile.Parcel.Leases[0].Leasees[0], '',
           //XMLFile.Parcel, 'LegalModeInfo/Grantor');
    end;

    // , 
    for I := 1 to 2 do
    begin

      if I = 1 then
        DescrName := 'LM' else
        DescrName := 'LS';

      for J := 0 to In4File.In4ObjectCount - 1 do
      begin
        In4Object := In4File.In4Objects[J];

        //     
        if not (In4Object.ObjectType in [chkin4Parcel, chkin4LandParcel]) or
          (GetIn4DescriptorValue(In4Object, DescrName, 1) = '') then Continue;

        //    In4
        Restrictions := SeparateString(GetIn4DescriptorValue(In4Object, DescrName, 1), '|');
        RestrictionCount := Length(Restrictions);

        for K := 1 to RestrictionCount do
        begin

          //    XML
          XMLRestriction := XMLFile.Parcel.AddNewRestriction;
          XMLRestriction.Geometry := In4Object.Geometry;

          //  /
          RestrictionCode := GetIn4DescriptorValue(In4Object, DescrName, 1, K);
          if DescrName = 'LM' then
          begin
            NewRestrictionCode := GetValueByKey(RestrictionCode, 'In4ToXMLRestrictionCode.txt');
            if NewRestrictionCode <> '' then RestrictionCode := NewRestrictionCode;
          end;
          XMLRestriction.SetValue('RestrictionCode', RestrictionCode);

          //  -  /
          if DescrName = 'LM' then
              DocType := GetValueByKey(GetIn4DescriptorValue(In4Object, DescrName, 3, K),
                'In4ToXMLRestrictionEntitlementDocType.txt')
            else
              DocType := GetValueByKey(GetIn4DescriptorValue(In4Object, DescrName, 2, K),
                'In4ToXMLRestrictionEntitlementDocType.txt');

          if DocType<>''
            then XMLRestriction.SetValue('RestrictionEntitlement/DocumentType', DocType);

          //   
          if DescrName='LS' then
          begin
            RestrictionTerm := GetIn4DescriptorValue(In4Object, DescrName, 3);
            if RestrictionTerm <> '' then
              XMLRestriction.SetValue('RestrictionTerm/Time/Duration',RestrictionTerm) else
              XMLRestriction.SetValue('RestrictionTerm/Permanent','');
          end;

          //   
          RegistrationDate := GetIn4DescriptorValue(In4Object, DescrName, 4, K);
          if RegistrationDate<>''
            then XMLRestriction.SetValue('RegistrationDate', In4ToXMLDate(RegistrationDate));

          //   
          RegistrationNumber := GetIn4DescriptorValue(In4Object, DescrName, 5, K);
          if RegistrationNumber<>''
            then XMLRestriction.SetValue('RegistrationNumber', RegistrationNumber);

          // 
          Beneficiaries := SeparateString(GetIn4DescriptorValue(In4Object, DescrName, 6), '|');
          for L := 0 to High(Beneficiaries) do
          begin

            //  
            XMLBeneficiary := XMLRestriction.AddNewBeneficiary;

            //  
            TaxNumber := Beneficiaries[L];
            IsNaturalPerson := Length(TaxNumber) = 10;
            if IsNaturalPerson
              then Path := 'NaturalPerson/TaxNumber'
              else Path := 'LegalEntity/EDRPOU';
            XMLBeneficiary.SetValue(Path, TaxNumber);

          end;

          //    
          if DescrName = 'LM' then
          begin
            MoneyPayment := GetIn4DescriptorValue(In4Object, DescrName, 7);
            if MoneyPayment <> '' then
              XMLRestriction.SetValue('Payment/Paid/MoneyPayment',MoneyPayment);
          end;

        end;
      end;
    end;

    for I := 0 to In4Parcel.LandParcelCount - 1 do
    begin

      //    XML
      In4LandParcel := In4Parcel.LandParcels[I];
      XMLLandParcel := XMLFile.Parcel.AddNewLandParcel;
      XMLLandParcel.Geometry := In4LandParcel.Geometry;

      //    
      XMLLandParcel.SetValue('CadastralCode', FullCadastralNumber);

      //  
      LandCode := GetValueByKey(GetIn4DescriptorValue(In4LandParcel, 'CN'), 'In4ToXMLLandCode.txt');
      XMLLandParcel.SetValue('LandCode', LandCode);

    end;

    for I := 0 to In4File.Quarter.NeighborCount - 1 do
    begin

      //    XML
      In4Neighbor := In4File.Quarter.Neighbors[I];
      XMLNeighbor := XMLFile.Parcel.AddNewNeighbor;
      XMLNeighbor.Geometry := In4Neighbor.Geometry;

      //  
      NeigbourNames := GetNeigbourNames(GetIn4DescriptorValue(In4Neighbor, 'NM'));
      for J := 0 to High(NeigbourNames) do
      begin
        XMLProprietor := XMLNeighbor.AddNewProprietor;
        if IsNeigbourNaturalPerson(NeigbourNames[J]) then
        begin
          NeigbourNames[J] := ClearNaturalPersonName(NeigbourNames[J]);
          ParseFullName(NeigbourNames[J], FirstName, MiddleName, LastName);
          XMLProprietor.SetValue('NaturalPerson/FullName/LastName', LastName);
          XMLProprietor.SetValue('NaturalPerson/FullName/FirstName', FirstName);
          if MiddleName<>''
            then XMLProprietor.SetValue('NaturalPerson/FullName/MiddleName', MiddleName);
        end else
          XMLProprietor.SetValue('LegalEntity/Name', NeigbourNames[J]);
      end;

    end;

    //   XML    ,    In4
    FileName := ChangeFileExt(In4File.FileName, '.xml');
    XMLFile.SaveToFile(FileName);

    //     In4
    App.MainWindow.FileTabs.CloseTab(FileTab.TabIndex);

    //   XML
    App.MainWindow.LoadFile(FileName);

  end;

var
  App: IchkApplication;
begin

  //      
  //       1.1
  CheckScriptVersion(1.1);

  //  -
  App := CreateApplication;
  //   
  App.InitLocalizer(App.GetThisScriptFolder + '\' + 'Messages.lng', 'ru');

  //     In4  XML
  ConvertActiveIn4ToXML(App);

end.